@@ -1,40 +1,25 @@
 """CORS Middleware"""
 
-from typing import List, Mapping, AbstractSet, Optional
 import logging
 import re
-from baretypes import (
-    Header,
-    Scope,
-    Info,
-    RouteMatches,
-    Content,
+from typing import AbstractSet, Iterable, List, Mapping, Optional, Tuple
+
+from bareasgi import (
     HttpRequestCallback,
+    HttpRequest,
     HttpResponse,
 )
-from bareutils import text_writer
-import bareutils.header as header
+from bareutils import text_writer, header
 
 ALL_METHODS = {"DELETE", "GET", "OPTIONS", "PATCH", "POST", "PUT"}
 
-ACCESS_CONTROL_ALLOW_CREDENTIALS = b"access-control-allow-credentials"
-ACCESS_CONTROL_ALLOW_HEADERS = b"access-control-allow-headers"
-ACCESS_CONTROL_ALLOW_METHODS = b"access-control-allow-methods"
-ACCESS_CONTROL_ALLOW_ORIGIN = b"access-control-allow-origin"
-ACCESS_CONTROL_EXPOSE_HEADERS = b"access-control-expose-headers"
-ACCESS_CONTROL_MAX_AGE = b"access-control-max-age"
-ACCESS_CONTROL_REQUEST_METHOD = b"access-control-request-method"
-ACCESS_CONTROL_REQUEST_HEADERS = b"access-control-request-headers"
-COOKIE = b"cookie"
-ORIGIN = b"origin"
-VARY = b"vary"
+LOGGER = logging.getLogger(__name__)
 
-logger = logging.getLogger(__name__)
+Header = Tuple[bytes, bytes]
 
 
 class CORSMiddleware:
-    """A CORS middleware implementation
-    """
+    """A CORS middleware implementation"""
 
     def __init__(
             self,
@@ -48,7 +33,7 @@ class CORSMiddleware:
             max_age: int = 600
     ) -> None:
         """Construct the CORS middleware
-        
+
         Args:
             allow_origins (Optional[AbstractSet[str]], optional): An optional
                 set of the allowed origins, or None for any origin.. Defaults
@@ -78,28 +63,34 @@ class CORSMiddleware:
         self.allow_all_origins = allow_origins is None
         if self.allow_all_origins:
             self.allow_origins = None
-            self.simple_headers.append((ACCESS_CONTROL_ALLOW_ORIGIN, b"*"))
+            self.simple_headers.append(
+                (header.ACCESS_CONTROL_ALLOW_ORIGIN, b"*"))
         else:
             self.allow_origins = allow_origins
 
         if allow_credentials:
             self.simple_headers.append(
-                (ACCESS_CONTROL_ALLOW_CREDENTIALS, b"true"))
+                (header.ACCESS_CONTROL_ALLOW_CREDENTIALS, b"true"))
 
         if expose_headers:
             self.simple_headers.append(
-                (ACCESS_CONTROL_EXPOSE_HEADERS, ", ".join(expose_headers).encode()))
+                (header.ACCESS_CONTROL_EXPOSE_HEADERS, ", ".join(expose_headers).encode()))
 
         self.preflight_headers: List[Header] = []
         if self.allow_all_origins:
-            self.preflight_headers.append((ACCESS_CONTROL_ALLOW_ORIGIN, b"*"))
+            self.preflight_headers.append(
+                (header.ACCESS_CONTROL_ALLOW_ORIGIN, b"*"))
         else:
-            self.preflight_headers.append((VARY, b"Origin"))
+            self.preflight_headers.append((header.VARY, b"Origin"))
 
         self.preflight_headers.append(
-            (ACCESS_CONTROL_ALLOW_METHODS, ", ".join(self.allow_methods).encode()))
+            (
+                header.ACCESS_CONTROL_ALLOW_METHODS,
+                ", ".join(self.allow_methods).encode()
+            )
+        )
         self.preflight_headers.append(
-            (ACCESS_CONTROL_MAX_AGE, str(max_age).encode())
+            (header.ACCESS_CONTROL_MAX_AGE, str(max_age).encode())
         )
 
         self.allow_all_headers = allow_headers is None
@@ -108,11 +99,16 @@ class CORSMiddleware:
         else:
             self.allow_headers = allow_headers
             self.preflight_headers.append(
-                (ACCESS_CONTROL_ALLOW_HEADERS, ", ".join(allow_headers).encode()))
+                (
+                    header.ACCESS_CONTROL_ALLOW_HEADERS,
+                    ", ".join(allow_headers).encode()
+                )
+            )
 
         if allow_credentials:
             self.preflight_headers.append(
-                (ACCESS_CONTROL_ALLOW_CREDENTIALS, b"true"))
+                (header.ACCESS_CONTROL_ALLOW_CREDENTIALS, b"true")
+            )
 
         self.allow_origin_regex = compiled_allow_origin_regex
 
@@ -120,10 +116,11 @@ class CORSMiddleware:
             self,
             request_header_map: Mapping[bytes, List[bytes]]
     ) -> HttpResponse:
-        response_headers: List[Header] = list(self.preflight_headers)
+        response_headers: List[Header] = list(
+            self.preflight_headers)
 
         try:
-            requested_origin = request_header_map[ORIGIN][0]
+            requested_origin = request_header_map[header.ORIGIN][0]
             if self._is_allowed_origin(origin=requested_origin):
                 if not self.allow_all_origins:
                     # If self.allow_all_origins is True, then the "Access-Control-Allow-Origin"
@@ -131,7 +128,7 @@ class CORSMiddleware:
                     # If we only allow specific origins, then we have to mirror back
                     # the Origin header in the response.
                     header.upsert(
-                        ACCESS_CONTROL_ALLOW_ORIGIN,
+                        header.ACCESS_CONTROL_ALLOW_ORIGIN,
                         requested_origin,
                         response_headers
                     )
@@ -140,17 +137,18 @@ class CORSMiddleware:
                     f'Invalid origin {requested_origin!r}'
                 )
 
-            requested_method = request_header_map[ACCESS_CONTROL_REQUEST_METHOD][0]
+            requested_method = request_header_map[header.ACCESS_CONTROL_REQUEST_METHOD][0]
             if requested_method.decode() not in self.allow_methods:
                 raise RuntimeError(f'Invalid method {requested_method!r}')
 
             # If we allow all headers, then we have to mirror back any requested
             # headers in the response.
-            if ACCESS_CONTROL_REQUEST_HEADERS in request_header_map:
-                access_control_request_header = request_header_map[ACCESS_CONTROL_REQUEST_HEADERS][0]
+            if header.ACCESS_CONTROL_REQUEST_HEADERS in request_header_map:
+                access_control_request_header = request_header_map[
+                    header.ACCESS_CONTROL_REQUEST_HEADERS][0]
                 if self.allow_all_headers:
                     header.upsert(
-                        ACCESS_CONTROL_ALLOW_HEADERS,
+                        header.ACCESS_CONTROL_ALLOW_HEADERS,
                         access_control_request_header,
                         response_headers
                     )
@@ -159,16 +157,16 @@ class CORSMiddleware:
                         if hdr.strip() not in self.allow_headers:
                             raise RuntimeError(f'Invalid header {hdr}')
 
-            logger.debug('Passed preflight checks')
+            LOGGER.debug('Passed preflight checks')
 
-            return 200, response_headers, text_writer("OK")
+            return HttpResponse(200, response_headers, text_writer("OK"))
 
         except RuntimeError as error:
-            logger.warning('Failed preflight checks with error %s', error)
+            LOGGER.warning('Failed preflight checks with error %s', error)
             # We don't strictly need to use 400 responses here, since its up to
             # the browser to enforce the CORS policy, but its more informative
             # if we do.
-            return 400, response_headers, text_writer(str(error))
+            return HttpResponse(400, response_headers, text_writer(str(error)))
 
     def _is_allowed_origin(self, origin: bytes) -> bool:
         if self.allow_all_origins:
@@ -185,31 +183,24 @@ class CORSMiddleware:
 
     async def _simple_response(
             self,
-            scope: Scope,
-            info: Info,
-            matches: RouteMatches,
-            content: Content,
+            request: HttpRequest,
             handler: HttpRequestCallback
     ) -> HttpResponse:
 
-        request_headers: List[Header] = scope['headers']
+        request_headers: Iterable[Header] = request.scope['headers']
 
-        status, headers, response_content, pushes = await handler(
-            scope,
-            info,
-            matches,
-            content
-        )
-        if headers is None:
-            headers = []
+        response = await handler(request)
 
-        origin = header.find(ORIGIN, request_headers)
+        headers = [] if response.headers is None else list(response.headers)
+
+        origin = header.find(header.ORIGIN, request_headers)
+        assert origin is not None
 
         # If request includes any cookie headers, then we must respond
         # with the specific origin instead of '*'.
-        if self.allow_all_origins and header.find(COOKIE, request_headers):
+        if self.allow_all_origins and header.find(header.COOKIE, request_headers):
             header.upsert(
-                ACCESS_CONTROL_ALLOW_ORIGIN,
+                header.ACCESS_CONTROL_ALLOW_ORIGIN,
                 origin,
                 self.simple_headers
             )
@@ -218,47 +209,46 @@ class CORSMiddleware:
         # the Origin header in the response.
         elif not self.allow_all_origins and self._is_allowed_origin(origin=origin):
             header.upsert(
-                ACCESS_CONTROL_ALLOW_ORIGIN,
+                header.ACCESS_CONTROL_ALLOW_ORIGIN,
                 origin,
                 headers
             )
-            vary_values = header.find(VARY, headers)
+            vary_values = header.find(header.VARY, headers)
             if not vary_values:
-                headers.append((VARY, b'Origin'))
+                headers.append((header.VARY, b'Origin'))
             else:
-                header.upsert(VARY, vary_values + b',Origin', headers)
+                header.upsert(header.VARY, vary_values + b',Origin', headers)
 
         for name, value in self.simple_headers:
             header.upsert(name, value, headers)
 
-        return status, headers, response_content, pushes
+        return HttpResponse(
+            response.status,
+            headers,
+            response.body,
+            response.pushes
+        )
 
     async def __call__(
             self,
-            scope: Scope,
-            info: Info,
-            matches: RouteMatches,
-            content: Content,
+            request: HttpRequest,
             handler: HttpRequestCallback
     ) -> HttpResponse:
-        headers = header.to_dict(scope['headers'])
+        headers = header.to_dict(request.scope['headers'])
 
-        if ORIGIN not in headers:
-            logger.debug(
+        if header.ORIGIN not in headers:
+            LOGGER.debug(
                 'CORS processsing skipped as there is no "%s" header',
-                ORIGIN
+                header.ORIGIN
             )
-            return await handler(scope, info, matches, content)
+            return await handler(request)
 
-        if scope["method"] == "OPTIONS" and ACCESS_CONTROL_REQUEST_METHOD in headers:
-            logger.debug('Performing preflight checks', extra=scope)
+        if (
+                request.scope["method"] == "OPTIONS" and
+                header.ACCESS_CONTROL_REQUEST_METHOD in headers
+        ):
+            LOGGER.debug('Performing preflight checks')
             return self._preflight_check(headers)
 
-        logger.debug('Processing simple response', extra=scope)
-        return await self._simple_response(
-            scope,
-            info,
-            matches,
-            content,
-            handler
-        )
+        LOGGER.debug('Processing simple response')
+        return await self._simple_response(request, handler)
